@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DHT_BUCKET_H_
-#define _D_DHT_BUCKET_H_
+#ifndef D_DHT_BUCKET_H
+#define D_DHT_BUCKET_H
 
 #include "common.h"
 
@@ -48,7 +48,6 @@
 namespace aria2 {
 
 class DHTNode;
-class Logger;
 
 class DHTBucket {
 private:
@@ -70,8 +69,6 @@ private:
 
   Timer lastUpdated_;
 
-  Logger* logger_;
-
   bool isInRange(const unsigned char* nodeID,
                  const unsigned char* max, const unsigned char* min) const;
 public:
@@ -157,4 +154,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_DHT_BUCKET_H_
+#endif // D_DHT_BUCKET_H
