@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_COMMON_H_
-#define _D_COMMON_H_
+#ifndef D_COMMON_H
+#define D_COMMON_H
 
 #ifdef __MINGW32__
 # undef SIZE_MAX
@@ -59,8 +59,24 @@ typedef _off_t off_t;
 #endif
 #endif // __MINGW32__
 
+#ifdef __MINGW32__
+# define WIN32_LEAN_AND_MEAN
+# ifndef WINVER
+#  define WINVER 0x501u
+# endif // WINVER
+# include <windows.h>
+#endif // __MINGW32__
+
 #ifdef ENABLE_NLS
-# include "gettext.h"
+// If we put #include <gettext.h> outside of #ifdef ENABLE_NLS and
+// --disable-nls is used, gettext(msgid) is defined as ((const char *)
+// (Msgid)). System header includes libintl.h regardless of
+// --disable-nls. For example, #include <string> will include
+// libintl.h through include chain. Since libintl.h refers gettext and
+// it is defined as non-function form, this causes compile error. User
+// reported gcc-4.2.2 has this problem. But gcc-4.4.5 does not suffer
+// from this problem.
+# include <gettext.h>
 # define _(String) gettext(String)
 #else // ENABLE_NLS
 # define _(String) String
@@ -71,4 +87,4 @@ typedef _off_t off_t;
 // included here for compatibility issues with old compiler/libraries.
 #include <stdint.h>
 
-#endif // _D_COMMON_H_
+#endif // D_COMMON_H
