@@ -9,6 +9,7 @@
 #include "Peer.h"
 #include "Option.h"
 #include "BtRuntime.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
@@ -24,6 +25,7 @@ class DefaultPeerStorageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testCalculateStat);
   CPPUNIT_TEST(testReturnPeer);
   CPPUNIT_TEST(testOnErasingPeer);
+  CPPUNIT_TEST(testAddBadPeer);
   CPPUNIT_TEST_SUITE_END();
 private:
   SharedHandle<BtRuntime> btRuntime;
@@ -47,13 +49,14 @@ public:
   void testCalculateStat();
   void testReturnPeer();
   void testOnErasingPeer();
+  void testAddBadPeer();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION(DefaultPeerStorageTest);
 
 void DefaultPeerStorageTest::testCountPeer() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
 
   CPPUNIT_ASSERT_EQUAL((size_t)0, ps.countPeer());
 
@@ -64,7 +67,7 @@ void DefaultPeerStorageTest::testCountPeer() {
 }
 
 void DefaultPeerStorageTest::testDeleteUnusedPeer() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
 
   SharedHandle<Peer> peer1(new Peer("192.168.0.1", 6889));
   SharedHandle<Peer> peer2(new Peer("192.168.0.2", 6889));
@@ -78,7 +81,7 @@ void DefaultPeerStorageTest::testDeleteUnusedPeer() {
 
   CPPUNIT_ASSERT_EQUAL((size_t)1, ps.countPeer());
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.3"),
-                       ps.getPeer("192.168.0.3", 6889)->ipaddr);
+                       ps.getPeer("192.168.0.3", 6889)->getIPAddress());
 
   ps.addPeer(peer1);
   ps.addPeer(peer2);
@@ -90,14 +93,14 @@ void DefaultPeerStorageTest::testDeleteUnusedPeer() {
   // peer2 has been in use, so it did't deleted.
   CPPUNIT_ASSERT_EQUAL((size_t)1, ps.countPeer());
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.2"),
-                       ps.getPeer("192.168.0.2", 6889)->ipaddr);
+                       ps.getPeer("192.168.0.2", 6889)->getIPAddress());
   
 }
 
 void DefaultPeerStorageTest::testAddPeer() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
   SharedHandle<BtRuntime> btRuntime(new BtRuntime());
-  btRuntime->setMaxPeers(3);
+  ps.setMaxPeerListSize(3);
   ps.setBtRuntime(btRuntime);
 
   SharedHandle<Peer> peer1(new Peer("192.168.0.1", 6889));
@@ -121,18 +124,40 @@ void DefaultPeerStorageTest::testAddPeer() {
   CPPUNIT_ASSERT(ps.addPeer(peer4));
   // peer2 was deleted. While peer1 is oldest, its cuid is not 0.
   CPPUNIT_ASSERT_EQUAL((size_t)3, ps.countPeer());
-  CPPUNIT_ASSERT(std::find(ps.getPeers().begin(), ps.getPeers().end(), peer2) == ps.getPeers().end());
+  CPPUNIT_ASSERT(std::find_if(ps.getPeers().begin(), ps.getPeers().end(),
+                              derefEqual(peer2)) == ps.getPeers().end());
 
   SharedHandle<Peer> peer5(new Peer("192.168.0.4", 0));
 
-  peer5->port = 6889;
+  peer5->setPort(6889);
 
-  // this returns false because the peer which has same ip and port has already added
+  // this returns false because the peer which has same ip and port
+  // has already added
   CPPUNIT_ASSERT_EQUAL(false, ps.addPeer(peer5));
+
+  SharedHandle<Peer> pa[] = {
+    SharedHandle<Peer>(new Peer("192.168.0.4", 6889)),
+    SharedHandle<Peer>(new Peer("192.168.0.5", 6889)),
+    SharedHandle<Peer>(new Peer("192.168.0.6", 6889)),
+    SharedHandle<Peer>(new Peer("192.168.0.7", 6889)),
+    SharedHandle<Peer>(new Peer("192.168.0.8", 6889))
+  };
+  std::vector<SharedHandle<Peer> > peers(vbegin(pa), vend(pa));
+  ps.addPeer(peers);
+  // peers[0] is not added because it has been already added.
+  // peers[1], peers[2] and peers[3] are going to be added.  peers[4]
+  // is not added because DefaultPeerStorage::addPeer() limits the
+  // number of peers to add.  Finally, unused peers are removed from
+  // back and size 3 vector is made.
+  CPPUNIT_ASSERT_EQUAL((size_t)3, ps.countPeer());
+  CPPUNIT_ASSERT(std::find_if(ps.getPeers().begin(), ps.getPeers().end(),
+                              derefEqual(peers[2])) != ps.getPeers().end());
+  CPPUNIT_ASSERT(std::find_if(ps.getPeers().begin(), ps.getPeers().end(),
+                              derefEqual(peers[3])) != ps.getPeers().end());
 }
 
 void DefaultPeerStorageTest::testGetUnusedPeer() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
   ps.setBtRuntime(btRuntime);
 
   SharedHandle<Peer> peer1(new Peer("192.168.0.1", 6889));
@@ -140,20 +165,20 @@ void DefaultPeerStorageTest::testGetUnusedPeer() {
   ps.addPeer(peer1);
 
   CPPUNIT_ASSERT_EQUAL(std::string("192.168.0.1"),
-                       ps.getUnusedPeer()->ipaddr);
+                       ps.getUnusedPeer()->getIPAddress());
 
   peer1->usedBy(1);
 
-  CPPUNIT_ASSERT(ps.getUnusedPeer().isNull());
+  CPPUNIT_ASSERT(!ps.getUnusedPeer());
 
   peer1->resetStatus();
   peer1->startBadCondition();
 
-  CPPUNIT_ASSERT(ps.getUnusedPeer().isNull());
+  CPPUNIT_ASSERT(!ps.getUnusedPeer());
 }
 
 void DefaultPeerStorageTest::testIsPeerAvailable() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
   ps.setBtRuntime(btRuntime);
 
   CPPUNIT_ASSERT_EQUAL(false, ps.isPeerAvailable());
@@ -176,10 +201,10 @@ void DefaultPeerStorageTest::testIsPeerAvailable() {
 }
 
 void DefaultPeerStorageTest::testActivatePeer() {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
 
   {
-    std::deque<SharedHandle<Peer> > peers;
+    std::vector<SharedHandle<Peer> > peers;
     ps.getActivePeers(peers);
     CPPUNIT_ASSERT_EQUAL((size_t)0, peers.size());
   }
@@ -189,7 +214,7 @@ void DefaultPeerStorageTest::testActivatePeer() {
   ps.addPeer(peer1);
 
   {
-    std::deque<SharedHandle<Peer> > activePeers;
+    std::vector<SharedHandle<Peer> > activePeers;
     ps.getActivePeers(activePeers);
 
     CPPUNIT_ASSERT_EQUAL((size_t)0, activePeers.size());
@@ -197,7 +222,7 @@ void DefaultPeerStorageTest::testActivatePeer() {
   {
     peer1->allocateSessionResource(1024*1024, 1024*1024*10);
 
-    std::deque<SharedHandle<Peer> > activePeers;
+    std::vector<SharedHandle<Peer> > activePeers;
     ps.getActivePeers(activePeers);
     
     CPPUNIT_ASSERT_EQUAL((size_t)1, activePeers.size());
@@ -209,25 +234,29 @@ void DefaultPeerStorageTest::testCalculateStat() {
 
 void DefaultPeerStorageTest::testReturnPeer()
 {
-  DefaultPeerStorage ps(option);
+  DefaultPeerStorage ps;
 
   SharedHandle<Peer> peer1(new Peer("192.168.0.1", 0));
   peer1->allocateSessionResource(1024*1024, 1024*1024*10);
   SharedHandle<Peer> peer2(new Peer("192.168.0.2", 6889));
   peer2->allocateSessionResource(1024*1024, 1024*1024*10);
   SharedHandle<Peer> peer3(new Peer("192.168.0.1", 6889));
+  peer2->setDisconnectedGracefully(true);
   ps.addPeer(peer1);
   ps.addPeer(peer2);
   ps.addPeer(peer3);
 
   ps.returnPeer(peer2); // peer2 removed from the container
   CPPUNIT_ASSERT_EQUAL((size_t)2, ps.getPeers().size());
-  CPPUNIT_ASSERT(std::find(ps.getPeers().begin(), ps.getPeers().end(), peer2)
-                 == ps.getPeers().end());
+  CPPUNIT_ASSERT(std::find_if(ps.getPeers().begin(), ps.getPeers().end(),
+                              derefEqual(peer2)) == ps.getPeers().end());
+  CPPUNIT_ASSERT_EQUAL((size_t)1, ps.getDroppedPeers().size());
 
   ps.returnPeer(peer1); // peer1 is removed from the container
   CPPUNIT_ASSERT_EQUAL((size_t)1, ps.getPeers().size());
-  CPPUNIT_ASSERT(std::find(ps.getPeers().begin(), ps.getPeers().end(), peer1) == ps.getPeers().end());
+  CPPUNIT_ASSERT(std::find_if(ps.getPeers().begin(), ps.getPeers().end(),
+                              derefEqual(peer1)) == ps.getPeers().end());
+  CPPUNIT_ASSERT_EQUAL((size_t)1, ps.getDroppedPeers().size());
 }
 
 void DefaultPeerStorageTest::testOnErasingPeer()
@@ -235,4 +264,12 @@ void DefaultPeerStorageTest::testOnErasingPeer()
   // test this
 }
 
+void DefaultPeerStorageTest::testAddBadPeer()
+{
+  DefaultPeerStorage ps;
+  ps.addBadPeer("192.168.0.1");
+  CPPUNIT_ASSERT(ps.isBadPeer("192.168.0.1"));
+  CPPUNIT_ASSERT(!ps.isBadPeer("192.168.0.2"));
+}
+
 } // namespace aria2
